/***************************************************************************                          
    file:           hdsensorslist.cpp
    description:    hard disk sensors list
    begin:          09/06/2024
    Copyright:      Giuseppe Della Bianca<giusdbg@gmail.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/types.h>
#include <stdlib.h>

#include <qstringlist.h>

#include <qregularexpression.h>
#include <qdebug.h>
#include <qfile.h>

#include <klocalizedstring.h>

#include "general.h"
#include "hdsensorslist.h"


HDSensorsList::HDSensorsList( SensorDevsList *sensorDevsList, const QString &name ): SensorDev( sensorDevsList, name )
{
 process = nullptr;

 setDescription( QString(i18n("Hard Disks")) );

 setClass(Sensor::hdSensor);

 readConfig();

 if(!isHddTempInstalled()) return;

 QStringList disks;
 if(!getDisks(disks)) return;

 for(QStringList::Iterator it = disks.begin(); it != disks.end(); ++it ) 
 {
    struct Sensor::T_ValData valData;

   ProcessExec proc;
   proc << qsFL1( "hddtemp" ) << qsFL1( "-q" );
   proc << *it;
   if(proc.runAndWait()) 
   {
     double value;
     QString str;
     if(getDiskInfo(proc.getStdoutData(),str,value)) 
     {
        valData.init();

        valData.val = value;
        valData.scale = Sensor::dgCelsius;
       
       Sensor *sensor= new Sensor( this );
       sensor->setType(Sensor::lmTemp);
       sensor->setObjectName(*it);
       sensor->setSubName ( str );
       sensor->setDescription(str);
       sensor->setValueMax  (40   , Sensor::dgCelsius);
       sensor->setValueMin  ( 0   , Sensor::dgCelsius);
       sensor->setValueIdeal(value, Sensor::dgCelsius);
       sensor->setValue( valData );
       sensor->readConfig();
     }
   }
 }
}

HDSensorsList::~HDSensorsList()
{
 delete process;
}

//*************

void HDSensorsList::updateSensors ( bool enabUpdate )
{
    bool withProcess;
//  qDebug() << "\033[91m" << "qDebug(): HDSensorsList::updateSensors start" << process <<"\033[0m";
 
    withProcess = false;
    
 const QObjectList *list= &children();
  
 if( !process && list)
 {

 QStringList params;  
 for ( QObjectList::ConstIterator it = list->begin(); it != list->end(); ++it ) {
    Sensor *obj = (Sensor *)(*it);
    
    if(obj->monitorized())
      params << obj->getLongName();
 }

 if(params.count()>0) 
 {
        withProcess = true;
   
   process= new ProcessExec;
   *process << qsFL1( "hddtemp" ) << qsFL1( "-q" ) << params;

//   connect( process, SIGNAL(finished(int)), this, SLOT(slotProcessExited(int)) );
   connect(process, QOverload<int, QProcess::ExitStatus>::of(&KProcess::finished), this, [this, enabUpdate]
        (int exitCode, QProcess::ExitStatus exitStatus) { this->slotProcessExited(exitCode, exitStatus, enabUpdate); });

//   qDebug() << "\033[91m" << "qDebug(): HDSensorsList::updateSensors run" << process <<"\033[0m";
   
   process->run();
 }
 
 }
 
    if ( !withProcess )
      setWithGlobalTimerUpdate ( false );
}

void HDSensorsList::slotProcessExited( int exitCode, QProcess::ExitStatus exitStatus, bool enabUpdate )
{
    Q_UNUSED( exitCode );
    Q_UNUSED( exitStatus );

    struct Sensor::T_ValData valData;

 const QObjectList *list= &children();

 if (list)
 {

    valData.init();
    valData.enabUpdate = enabUpdate;
 
 if (process->outputErrors())
   qWarning() << "qWarning(): HDSensorsList::slotProcessExited HddTemp Error: " << process->getStdoutData();

 QStringList buf = process->getStdoutData().split( QChar::fromLatin1( '\n' ) );
 for(QStringList::Iterator it = buf.begin(); it != buf.end(); ++it ) {
   for ( QObjectList::ConstIterator olIt = list->begin(); olIt != list->end(); ++olIt ) {
     Sensor *obj = (Sensor *)(*olIt);

     QRegularExpression rx(QString( obj->getLongName() ) + qsFL1( ":\\s+.+:\\s+(\\d+).*C" ) );

     QRegularExpressionMatch qremHDD = rx.match( *it );

     if ( qremHDD.hasMatch() )
     {
          valData.val = qremHDD.captured(1).toDouble();
          valData.scale = Sensor::dgCelsius;

          obj->setValue( valData );
     }
   }
 }
 
 // qDebug() << "\033[91m" << "qDebug(): HDSensorsList::slotProcessExited delete" << process <<"\033[0m";
 
// delete process;
 process->deleteLater();
 process = nullptr;
 
 }
 
 setWithGlobalTimerUpdate ( false );
 
 // qDebug() << "\033[91m" << "qDebug(): HDSensorsList::slotProcessExited post delete" << process <<"\033[0m";
}

// ***************  Static methods


bool HDSensorsList::getDiskInfo(const QString buf, QString &name, double &value)
{
 QRegularExpression rx( qsFL1( ":\\s+(.+):\\s+(\\d+).*C" ) );

    QRegularExpressionMatch qremHDD = rx.match( buf );
 
 if ( qremHDD.hasMatch()) 
 {
   bool ok;
   name = qremHDD.captured(1);
   value = qremHDD.captured(2).toDouble(&ok);
   if (ok)
     return true;
   else
     return false;
 }
 else 
   return false;
}


bool HDSensorsList::isHddTempInstalled()
{
 ProcessExec proc;

 proc << qsFL1( "hddtemp" ) << qsFL1( "-v" );
 if(proc.runAndWait()) {
   if(proc.getStdoutData().contains( qsFL1( "ERROR" ) )==0 ) return true;
   qWarning() << "qWarning(): HDSensorsList::isHddTempInstalled HddTemp Error: " << proc.getStdoutData();  
 }
 return false;
}

bool HDSensorsList::getDisks(QStringList &disks )
{
  DIR *dir;

  /* Get a listing of the hard drives looking under sysfs first then falling back to /proc/ide */
  if((dir = opendir ("/sys/block")) == NULL)
    if ((dir = opendir ("/proc/ide")) == NULL)
      return false;
  QString str;
  struct dirent *ptr;
  while((ptr= readdir(dir))) {
    if((ptr->d_name[0]=='h' || ptr->d_name[0]=='s') && ptr->d_name[1]=='d') {
       str = qsFL1( "/dev/%1" ).arg( qsFL1( ptr->d_name ) );
       disks << str;
    }
  }
  closedir(dir);
  return true;
}

// ***************

