/***************************************************************************                          
    file:           syssensorslist.cpp
    description:    system sensors list
    begin:          09/06/2024
    Copyright:      Giuseppe Della Bianca<giusdbg@gmail.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qglobal.h>

#include <klocalizedstring.h>

#include "general.h"
#include "procinfo.h"
#include "syssensorslist.h"

SYSSensorsList::SYSSensorsList( SensorDevsList *sensorDevsList, const QString &name ): SensorDev( sensorDevsList, name )
{
    struct Sensor::T_ValData valData;

    getCpuTime( &old_user,&old_nice,&old_system,&old_idle );

    valData.init();
    valData.val = 30;
    valData.scale = Sensor::dgNoTemp;

    setDescription( QString ( i18n ( "System Information" ) ) );

    setClass( Sensor::sysSensor );

    readConfig();

    sCpuInfo = new Sensor( this );
    sCpuInfo->setType( Sensor::sysCpuInfo );
    sCpuInfo->setObjectName( qsFL1( "proc.CPUINFO" ) );
    sCpuInfo->setDescription(  qsFL1( "CPU Info" ) );
    sCpuInfo->setValueMax( -1000000, Sensor::dgNoTemp );
    sCpuInfo->setValueMin( -1000000, Sensor::dgNoTemp );
    sCpuInfo->setValueIdeal( 30.0, Sensor::dgNoTemp );
    sCpuInfo->setValue( valData );
    sCpuInfo->readConfig();

    sCpuLoad = new Sensor( this );
    sCpuLoad->setType( Sensor::sysCpuLoad );
    sCpuLoad->setObjectName( qsFL1( "proc.CPULOAD" ) );
    sCpuLoad->setDescription( qsFL1( "CPU Load" ) );
    sCpuLoad->setValueMax( -1000000, Sensor::dgNoTemp );
    sCpuLoad->setValueMin( -1000000, Sensor::dgNoTemp );
    sCpuLoad->setValueIdeal( 30.0, Sensor::dgNoTemp );
    sCpuLoad->setValue( valData );
    sCpuLoad->readConfig();

    sRamUsed = new Sensor( this );
    sRamUsed->setType( Sensor::sysRamUsed );
    sRamUsed->setObjectName( qsFL1( "proc.RAMINFO" ) );
    sRamUsed->setDescription( qsFL1( "RAM Used" ) );
    sRamUsed->setValueMax( -1000000, Sensor::dgNoTemp );
    sRamUsed->setValueMin( -1000000, Sensor::dgNoTemp );
    sRamUsed->setValueIdeal( 30.0, Sensor::dgNoTemp );
    sRamUsed->setValue( valData );
    sRamUsed->readConfig();

    sSwapUsed = new Sensor( this );
    sSwapUsed->setType( Sensor::sysSwapUsed );
    sSwapUsed->setObjectName( qsFL1( "proc.SWAPINFO" ) );
    sSwapUsed->setDescription( qsFL1( "SWAP Used" ) );
    sSwapUsed->setValueMax( -1000000, Sensor::dgNoTemp );
    sSwapUsed->setValueMin( -1000000, Sensor::dgNoTemp );
    sSwapUsed->setValueIdeal( 30.0, Sensor::dgNoTemp );
    sSwapUsed->setValue( valData );
    sSwapUsed->readConfig();

    sUpTime = new Sensor( this );
    sUpTime->setType( Sensor::sysUpTime );
    sUpTime->setObjectName( qsFL1( "proc.UPTIME" ) );
    sUpTime->setDescription( qsFL1( "Up Time" ) );
    sUpTime->setValueMax( -1000000, Sensor::dgNoTemp );
    sUpTime->setValueMin( -1000000, Sensor::dgNoTemp );
    sUpTime->setValueIdeal( 30.0, Sensor::dgNoTemp );
    sUpTime->setValue( valData );
    sUpTime->setValDescr( qsFL1( "00:00" ) );
    sUpTime->setDescr2( qsFL1( "hh:mm" ) );
    sUpTime->readConfig();
    
    updateSensors ( false );
}

SYSSensorsList::~SYSSensorsList()
{
}

void SYSSensorsList::updateSensors ( bool enabUpdate )
{
    QString sCpu,sVendor,sBogomips,sSpeed;
    int user, nice, system;
    int newRamTotal,newRamUsed,newSwapTotal,newSwapUsed,newMemTotal,newMemUsed;
    int idle, uptime;

    struct Sensor::T_ValData valInfo, valLoad, valRam, valSwap, valUpTime;
    
    QString str1, str2;

    valInfo.init();
    valLoad.init(); 
    valRam.init();
    valSwap.init();
    valUpTime.init();

    valInfo.enabUpdate =  valLoad.enabUpdate = valRam.enabUpdate = valSwap.enabUpdate = valUpTime.enabUpdate = enabUpdate;

    getCpuInfoValue( "model name", sCpu );
    getCpuInfoValue( "vendor_id", sVendor );
    getCpuInfoValue( "bogomips", sBogomips );
    getCpuInfoValue( "cpu MHz", sSpeed );
    
    getCpuTime( &user, &nice, &system, &idle );
    
    getMemInfo( &newRamTotal, &newRamUsed, &newSwapTotal, &newSwapUsed );
    
    uptime = getUpTime();


    adjustString( sCpu, 16 );
    adjustString( sVendor, 16 );
    str1 = sCpu;
    str2 = qsFL1( "%1 BMPS" ).arg ( sBogomips.toDouble(), 0, 'f', 0, QChar::fromLatin1( ' ' ) );

    valInfo.val = sSpeed.toDouble();
    
    sCpuInfo->setValue ( valInfo );
    sCpuInfo->setDescr1( str1 );
    sCpuInfo->setDescr2( str2 );

    
    int puser  = user   - old_user;
    int pnice  = nice   - old_nice;
    int psystem= system - old_system;
    int pidle  = idle   - old_idle;

    int ptotal=  puser+pnice+psystem+pidle;

    if (ptotal==0) ptotal=1;

    // ptotal value, puser value1, pnice value2, psystem value3, pidle value4

    valLoad.val = ( ( 1 - ( (double)pidle / ptotal ) ) * 100.0 );
    valLoad.val1 = ( (double)puser / ptotal * 100.0 );
    valLoad.val2 = ( (double)pnice / ptotal * 100.0 );
    valLoad.val3 = ( (double)psystem / ptotal * 100.0 ), 
    valLoad.val4 = ( (double)pidle / ptotal * 100.0 );
    valLoad.scale = Sensor::dgNoTemp;

    sCpuLoad->setValue( valLoad ); 
    
    old_user  = user;
    old_nice  = nice;
    old_system= system;
    old_idle  = idle;


    newMemTotal = qRound ( newRamTotal / 1024.0 );
    newMemUsed = qRound (newRamUsed / 1024.0 );

    sRamUsed->setValueMin ( 0.0 );
    sRamUsed->setValueMax ( newMemTotal );

    valRam.val = newMemUsed;
    valRam.val1 = newMemTotal;

    sRamUsed->setValue ( valRam );

    newMemTotal = qRound ( newSwapTotal / 1024.0 );
    newMemUsed = qRound ( newSwapUsed / 1024.0 );

    valSwap.val = newMemUsed;
    valSwap.val1 = newMemTotal;
    
    sSwapUsed->setValueMin ( 0.0 );
    sSwapUsed->setValueMax ( newMemTotal );
    sSwapUsed->setValue ( valSwap );


    int hours  = uptime/ (60*60);
    int minutes= (uptime % (60*60)) / 60;
    
    str1 = qsFL1( "%1:%2" ).arg( hours, 2, 10,  QChar::fromLatin1 ( '0' ) ).arg( minutes, 2, 10, QChar::fromLatin1 ( '0' ) );
    
    valUpTime.val = uptime;
    valUpTime.scale = Sensor::dgNoTemp;
    
    sUpTime->setValue( valUpTime );
    sUpTime->setValDescr( str1 );

    setWithGlobalTimerUpdate ( false );

} // void updateSensors()
